# MegaRAID Helper
module MegaRaid
  cli_path = node.megaraid.install_path
  _cli_exec = cli_path + '/' + node.megaraid.exec_name
  # Static Definitions
  module Statics
    _disk_rules = {
      'raid' => {
        'key' => 'level',
        '0' => {
          'minDisks' => 2,
          'maxDisks' => nil,
          'modDisks' => 1,
          'raise' => 'RAID0 is highly volatile, should be used with caution. To fix Chef runs please add \"warn\": false to your RAID0 config'
        },
        '1' => {
          'minDisks' => 2,
          'maxDisks' => 2,
          'modDisks' => 2
        },
        '10' => {
          'minDisks' => 4,
          'maxDisks' => 4,
          'modDisks' => 4
        },
        '5' => {
          'minDisks' => 3,
          'maxDisks' => nil,
          'modDisks' => 1
        },
        '6' => {
          'minDisks' => 4,
          'maxDisks' => nil,
          'modDisks' => 1
        },
        '50' => {
          'minDisks' => 6,
          'maxDisks' => nil,
          'modDisks' => 2
        },
        '60' => {
          'minDisks' => 8,
          'maxDisks' => nil,
          'modDisks' => 2
        }
      },
      'jbod' => {
        'key' => nil,
        'minDisks' => 1,
        'maxDisks' => nil,
        'modDisks' => 1
      }
    }
  end
  # Chef Helper Module
  module Helper
    def disks_unclean(_disks) # to be implemented later
      false
    end

    def already_configured_as_such(cfg)
      # Needs proper implementation, we're just safefty netting for already configured disks.
      cmd_str = "/opt/MegaRAID/MegaCli/MegaCli64 -PdInfo -PhysDrv [#{lookup_phys_drive(cfg.adapter, cfg.disks)}] -a#{cfg.adapter} | grep Span"
      cmd = Mixlib::ShellOut.new(cmd_str).run_command
      !cmd.stdout.split("\n").length.empty?
      # true
    end

    def validate_adapter_ids(cfg)
      cmd = Mixlib::ShellOut.new('/opt/MegaRAID/MegaCli/MegaCli64 -adpCount | grep Cont | sed -e "s/[^0-9]//g"').run_command
      total_adapters = Integer(cmd.stdout)
      cfg.each do |subcf|
        raise "Invalid Adapter Number! Configuration Invalid! (#{subcf})" if subcf.adapter >= total_adapters
      end
    end

    def validate_disk_ids(cfg)
      cfg.each do |subcf|
        cmd_str = "/opt/MegaRAID/MegaCli/MegaCli64 -PDList -a#{subcf.adapter} | grep -E \"Slot Number|PD Type\" | grep -E \"SATA|SAS\" -B1 | grep -v -E \"SATA|SAS\""
        cmd = Mixlib::ShellOut.new(cmd_str).run_command
        subcf.disks.each do |disk|
          raise "Invalid Disk Number (#{disk})! Configuration Invalid! (#{subcf})" unless cmd.stdout.index(String(disk))
        end
      end
    end

    def parse_drives(adapter)
      cmd_str = "#{MegaRaid.cli_exec} -PDList -a#{adapter} | grep -E \"Slot Number|PD Type|Enclosure Device ID\" | grep -E \"SATA|SAS\" -B2 | grep -v -E \"SATA|SAS\""
      cmd = Mixlib::ShellOut.new(cmd_str).run_command
      raise "Invalid output from MegaCLI for -PDList on -a#{adapter}." unless cmd.stdout.split("\n").length.even?
      raise "Invalid output from MegaCLI for -PDList on -a#{adapter}." unless cmd.stdout.scan(/Enclosure/).count == cmd.stdout.scan(/Slot/).count
      parsed_drives = []
      cmd.stdout.split("\n").length.times do |counter|
        parsed_drives[counter / 2] = [] if counter.even?
        parsed_drives[counter / 2][counter % 2] = cmd.stdout.split("\n")[counter].gsub(/\D/, '')
      end
      parsed_drives
    end

    def lookup_phys_drive(adapter, drives)
      post_drives = []
      parsed_drives = parse_drives adapter
      drives.each do |drive|
        post_drives << "#{parsed_drives[drive][0]}:#{parsed_drives[drive][1]}"
      end
      post_drives.join(',')
    end

    def exec_mega_cli(args)
      # puts "MJS - Would Execute: #{node['megaraid']['install_path']}/#{node['megaraid']['exec_name']} #{args}"
      cmd = Mixlib::ShellOut.new("#{node.megaraid.install_path}/#{node.megaraid.exec_name} #{args}").run_command
      cmd.exitstatus == 0
      # false
    end

    def parse_adapter(cfg, adapters)
      adapter = {}
      adapter["a#{cfg['adapter']}"] = { disks: {} }
      cfg['disks'].each do |disk|
        dsk = {}
        dsk["d#{disk}"] = true
        raise "Error: Disk #{disk} on -a#{cfg['adapter']} is in use" unless adapters.key?("a#{cfg['adapter']}") && adapters["a#{cfg['adapter']}"][:disks].key?("d#{disk}")
        adapter["a#{cfg['adapter']}"][:disks].merge! dsk
      end
      adapter
    end

    def check_for_conflicts(config)
      adapters = {}
      config.each do |cfg|
        adapter = parse_adapter(cfg, adapters)
        adapters.merge! adapter
      end
      # puts "MJS - Adapters: #{adapters}"
    end

    def check_raid_types(config)
      config.each do |cfg|
        diskrules = MegaRaid::Statics._disk_rules[cfg['type']]
        key = disk_rules[cfg['type']]['key']
        diskrules = diskrules[cfg[key]] unless key.nil?
        raise "Not enough Disks in #{cfg['type']}#{cfg[key]} On Adapter #{cfg['adapter']} with Disks: #{cfg[:disks]}" if cfg.disks.size < diskrules.minDisks
        if cfg['disks'].size % diskrules['modDisks'] > 0 || (!diskrules.maxDisks.nil? && cfg.disks.size > diskrules.maxDisks)
          raise "Incorrect number of Disks in #{cfg['type']}#{cfg[key]} On Adapter #{cfg['adapter']} with Disks: #{cfg[:disks]}"
        end
      end
    end
  end
end
